@@ -186,10 +186,10 @@ void ClipboardMap::SetLastModifiedTimeWithoutRunningCallback(base::Time time) {
 
 // Add a key:jstr pair to map, if jstr is null or is empty, then remove that
 // entry.
-void AddMapEntry(JNIEnv* env,
-                 std::map<std::string, std::string>* map,
-                 const char* key,
-                 const ScopedJavaLocalRef<jstring>& jstr) {
+void JNI_Clipboard_AddMapEntry(JNIEnv* env,
+                               std::map<std::string, std::string>* map,
+                               const char* key,
+                               const ScopedJavaLocalRef<jstring>& jstr) {
   if (jstr.is_null()) {
     map->erase(key);
     return;
@@ -223,8 +223,8 @@ void ClipboardMap::UpdateFromAndroidClipboard() {
   ScopedJavaLocalRef<jstring> jhtml =
       Java_Clipboard_getHTMLText(env, clipboard_manager_);
 
-  AddMapEntry(env, &map_, kPlainTextFormat, jtext);
-  AddMapEntry(env, &map_, kHTMLFormat, jhtml);
+  JNI_Clipboard_AddMapEntry(env, &map_, kPlainTextFormat, jtext);
+  JNI_Clipboard_AddMapEntry(env, &map_, kHTMLFormat, jhtml);
 
   map_state_ = MapState::kUpToDate;
 }
@@ -452,9 +452,10 @@ SkBitmap ClipboardAndroid::ReadImage(ClipboardType type) const {
 
     bmp.allocN32Pixels(size->width(), size->height());
 
-    DCHECK_EQ(sizeof(gfx::Size) + bmp.getSize(), input.size());
+    DCHECK_EQ(sizeof(gfx::Size) + bmp.computeByteSize(), input.size());
 
-    memcpy(bmp.getPixels(), input.data() + sizeof(gfx::Size), bmp.getSize());
+    memcpy(bmp.getPixels(), input.data() + sizeof(gfx::Size),
+           bmp.computeByteSize());
   }
   return bmp;
 }
@@ -540,7 +541,7 @@ void ClipboardAndroid::WriteBitmap(const SkBitmap& bitmap) {
 
   std::string packed(reinterpret_cast<const char*>(&size), sizeof(size));
   packed += std::string(static_cast<const char*>(bitmap.getPixels()),
-                        bitmap.getSize());
+                        bitmap.computeByteSize());
   g_map.Get().Set(kBitmapFormat, packed);
 }
 
@@ -550,13 +551,10 @@ void ClipboardAndroid::WriteData(const Clipboard::FormatType& format,
   g_map.Get().Set(format.ToString(), std::string(data_data, data_len));
 }
 
-bool RegisterClipboardAndroid(JNIEnv* env) {
-  return RegisterNativesImpl(env);
-}
-
 // Returns a pointer to the current ClipboardAndroid object.
-static jlong Init(JNIEnv* env,
-                  const base::android::JavaParamRef<jobject>& obj) {
+static jlong JNI_Clipboard_Init(
+    JNIEnv* env,
+    const base::android::JavaParamRef<jobject>& obj) {
   return reinterpret_cast<intptr_t>(Clipboard::GetForCurrentThread());
 }
 
