@@ -13,17 +13,18 @@
 #include "base/json/json_writer.h"
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
+#include "base/numerics/ranges.h"
 #include "base/trace_event/trace_event.h"
-#include "cc/base/filter_operation.h"
-#include "cc/base/filter_operations.h"
 #include "cc/layers/nine_patch_layer.h"
 #include "cc/layers/picture_layer.h"
 #include "cc/layers/solid_color_layer.h"
 #include "cc/layers/surface_layer.h"
 #include "cc/layers/texture_layer.h"
-#include "cc/output/copy_output_request.h"
-#include "cc/resources/transferable_resource.h"
+#include "cc/paint/filter_operation.h"
+#include "cc/paint/filter_operations.h"
 #include "cc/trees/layer_tree_settings.h"
+#include "components/viz/common/frame_sinks/copy_output_request.h"
+#include "components/viz/common/resources/transferable_resource.h"
 #include "ui/compositor/compositor_switches.h"
 #include "ui/compositor/dip_util.h"
 #include "ui/compositor/layer_animator.h"
@@ -40,6 +41,10 @@
 namespace {
 
 const ui::Layer* GetRoot(const ui::Layer* layer) {
+  // Parent walk cannot be done on a layer that is being used as a mask. Get the
+  // layer to which this layer is a mask of.
+  if (layer->layer_mask_back_link())
+    layer = layer->layer_mask_back_link();
   while (layer->parent())
     layer = layer->parent();
   return layer;
@@ -69,8 +74,8 @@ class Layer::LayerMirror : public LayerDelegate, LayerObserver {
     if (auto* delegate = source_->delegate())
       delegate->OnPaintLayer(context);
   }
-  void OnDelegatedFrameDamage(const gfx::Rect& damage_rect_in_dip) override {}
-  void OnDeviceScaleFactorChanged(float device_scale_factor) override {}
+  void OnDeviceScaleFactorChanged(float old_device_scale_factor,
+                                  float new_device_scale_factor) override {}
 
   // LayerObserver:
   void LayerDestroyed(Layer* layer) override {
@@ -87,8 +92,8 @@ class Layer::LayerMirror : public LayerDelegate, LayerObserver {
 
 Layer::Layer()
     : type_(LAYER_TEXTURED),
-      compositor_(NULL),
-      parent_(NULL),
+      compositor_(nullptr),
+      parent_(nullptr),
       visible_(true),
       fills_bounds_opaquely_(true),
       fills_bounds_completely_(false),
@@ -98,24 +103,24 @@ Layer::Layer()
       layer_grayscale_(0.0f),
       layer_inverted_(false),
       layer_blur_sigma_(0.0f),
-      layer_temperature_(0.0f),
-      layer_blue_scale_(1.0f),
-      layer_green_scale_(1.0f),
-      layer_mask_(NULL),
-      layer_mask_back_link_(NULL),
+      layer_mask_(nullptr),
+      layer_mask_back_link_(nullptr),
       zoom_(1),
       zoom_inset_(0),
-      delegate_(NULL),
-      owner_(NULL),
-      cc_layer_(NULL),
-      device_scale_factor_(1.0f) {
+      delegate_(nullptr),
+      owner_(nullptr),
+      cc_layer_(nullptr),
+      device_scale_factor_(1.0f),
+      cache_render_surface_requests_(0),
+      deferred_paint_requests_(0),
+      trilinear_filtering_request_(0) {
   CreateCcLayer();
 }
 
 Layer::Layer(LayerType type)
     : type_(type),
-      compositor_(NULL),
-      parent_(NULL),
+      compositor_(nullptr),
+      parent_(nullptr),
       visible_(true),
       fills_bounds_opaquely_(true),
       fills_bounds_completely_(false),
@@ -125,17 +130,17 @@ Layer::Layer(LayerType type)
       layer_grayscale_(0.0f),
       layer_inverted_(false),
       layer_blur_sigma_(0.0f),
-      layer_temperature_(0.0f),
-      layer_blue_scale_(1.0f),
-      layer_green_scale_(1.0f),
-      layer_mask_(NULL),
-      layer_mask_back_link_(NULL),
+      layer_mask_(nullptr),
+      layer_mask_back_link_(nullptr),
       zoom_(1),
       zoom_inset_(0),
-      delegate_(NULL),
-      owner_(NULL),
-      cc_layer_(NULL),
-      device_scale_factor_(1.0f) {
+      delegate_(nullptr),
+      owner_(nullptr),
+      cc_layer_(nullptr),
+      device_scale_factor_(1.0f),
+      cache_render_surface_requests_(0),
+      deferred_paint_requests_(0),
+      trilinear_filtering_request_(0) {
   CreateCcLayer();
 }
 
@@ -148,46 +153,47 @@ Layer::~Layer() {
   // is still around.
   SetAnimator(nullptr);
   if (compositor_)
-    compositor_->SetRootLayer(NULL);
+    compositor_->SetRootLayer(nullptr);
   if (parent_)
     parent_->Remove(this);
   if (layer_mask_)
-    SetMaskLayer(NULL);
+    SetMaskLayer(nullptr);
   if (layer_mask_back_link_)
-    layer_mask_back_link_->SetMaskLayer(NULL);
-  for (size_t i = 0; i < children_.size(); ++i)
-    children_[i]->parent_ = NULL;
+    layer_mask_back_link_->SetMaskLayer(nullptr);
+  for (auto* child : children_)
+    child->parent_ = nullptr;
 
   cc_layer_->RemoveFromParent();
-  if (mailbox_release_callback_)
-    mailbox_release_callback_->Run(gpu::SyncToken(), false);
+  if (transfer_release_callback_)
+    transfer_release_callback_->Run(gpu::SyncToken(), false);
 }
 
 std::unique_ptr<Layer> Layer::Clone() const {
-  auto clone = base::MakeUnique<Layer>(type_);
+  auto clone = std::make_unique<Layer>(type_);
 
   // Background filters.
   clone->SetBackgroundBlur(background_blur_sigma_);
   clone->SetBackgroundZoom(zoom_, zoom_inset_);
 
   // Filters.
-  clone->SetLayerTemperature(GetTargetTemperature());
   clone->SetLayerSaturation(layer_saturation_);
   clone->SetLayerBrightness(GetTargetBrightness());
   clone->SetLayerGrayscale(GetTargetGrayscale());
   clone->SetLayerInverted(layer_inverted_);
   clone->SetLayerBlur(layer_blur_sigma_);
   if (alpha_shape_)
-    clone->SetAlphaShape(base::MakeUnique<SkRegion>(*alpha_shape_));
+    clone->SetAlphaShape(std::make_unique<ShapeRects>(*alpha_shape_));
 
   // cc::Layer state.
   if (surface_layer_) {
-    if (surface_layer_->primary_surface_info().is_valid()) {
-      clone->SetShowPrimarySurface(surface_layer_->primary_surface_info(),
+    if (surface_layer_->primary_surface_id().is_valid()) {
+      clone->SetShowPrimarySurface(surface_layer_->primary_surface_id(),
+                                   frame_size_in_dip_,
+                                   surface_layer_->background_color(),
                                    surface_layer_->surface_reference_factory());
     }
-    if (surface_layer_->fallback_surface_info().is_valid())
-      clone->SetFallbackSurface(surface_layer_->fallback_surface_info());
+    if (surface_layer_->fallback_surface_id().is_valid())
+      clone->SetFallbackSurfaceId(surface_layer_->fallback_surface_id());
   } else if (type_ == LAYER_SOLID_COLOR) {
     clone->SetColor(GetTargetColor());
   }
@@ -207,7 +213,7 @@ std::unique_ptr<Layer> Layer::Clone() const {
 
 std::unique_ptr<Layer> Layer::Mirror() {
   auto mirror = Clone();
-  mirrors_.emplace_back(base::MakeUnique<LayerMirror>(this, mirror.get()));
+  mirrors_.emplace_back(std::make_unique<LayerMirror>(this, mirror.get()));
   return mirror;
 }
 
@@ -279,7 +285,7 @@ void Layer::Remove(Layer* child) {
       std::find(children_.begin(), children_.end(), child);
   DCHECK(i != children_.end());
   children_.erase(i);
-  child->parent_ = NULL;
+  child->parent_ = nullptr;
   child->cc_layer_->RemoveFromParent();
 }
 
@@ -315,7 +321,7 @@ void Layer::SetAnimator(LayerAnimator* animator) {
   Compositor* compositor = GetCompositor();
 
   if (animator_) {
-    if (compositor)
+    if (compositor && !layer_mask_back_link())
       animator_->DetachLayerAndTimeline(compositor);
     animator_->SetDelegate(nullptr);
   }
@@ -324,7 +330,7 @@ void Layer::SetAnimator(LayerAnimator* animator) {
 
   if (animator_) {
     animator_->SetDelegate(this);
-    if (compositor)
+    if (compositor && !layer_mask_back_link())
       animator_->AttachLayerAndTimeline(compositor);
   }
 }
@@ -386,18 +392,6 @@ float Layer::GetCombinedOpacity() const {
   return opacity;
 }
 
-void Layer::SetLayerTemperature(float value) {
-  GetAnimator()->SetTemperature(value);
-}
-
-float Layer::GetTargetTemperature() const {
-  if (animator_ &&
-      animator_->IsAnimatingProperty(LayerAnimationElement::TEMPERATURE)) {
-    return animator_->GetTargetTemperature();
-  }
-  return layer_temperature();
-}
-
 void Layer::SetBackgroundBlur(float blur_sigma) {
   background_blur_sigma_ = blur_sigma;
 
@@ -445,20 +439,19 @@ void Layer::SetLayerInverted(bool inverted) {
 }
 
 void Layer::SetMaskLayer(Layer* layer_mask) {
-  // The provided mask should not have a layer mask itself.
-  DCHECK(!layer_mask ||
-         (!layer_mask->layer_mask_layer() &&
-          layer_mask->children().empty() &&
-          !layer_mask->layer_mask_back_link_));
-  DCHECK(!layer_mask_back_link_);
   if (layer_mask_ == layer_mask)
     return;
+  // The provided mask should not have a layer mask itself.
+  DCHECK(!layer_mask ||
+         (!layer_mask->layer_mask_layer() && layer_mask->children().empty() &&
+          !layer_mask->layer_mask_back_link_));
+  DCHECK(!layer_mask_back_link_);
   // We need to de-reference the currently linked object so that no problem
   // arises if the mask layer gets deleted before this object.
   if (layer_mask_)
-    layer_mask_->layer_mask_back_link_ = NULL;
+    layer_mask_->layer_mask_back_link_ = nullptr;
   layer_mask_ = layer_mask;
-  cc_layer_->SetMaskLayer(layer_mask ? layer_mask->cc_layer_ : NULL);
+  cc_layer_->SetMaskLayer(layer_mask ? layer_mask->cc_layer_ : nullptr);
   // We need to reference the linked object so that it can properly break the
   // link to us when it gets deleted.
   if (layer_mask) {
@@ -474,8 +467,8 @@ void Layer::SetBackgroundZoom(float zoom, int inset) {
   SetLayerBackgroundFilters();
 }
 
-void Layer::SetAlphaShape(std::unique_ptr<SkRegion> region) {
-  alpha_shape_ = std::move(region);
+void Layer::SetAlphaShape(std::unique_ptr<ShapeRects> shape) {
+  alpha_shape_ = std::move(shape);
 
   SetLayerFilters();
 }
@@ -490,15 +483,6 @@ void Layer::SetLayerFilters() {
     filters.Append(cc::FilterOperation::CreateGrayscaleFilter(
         layer_grayscale_));
   }
-  if (layer_temperature_) {
-    float color_matrix[] = {
-        1.0f,               0.0f,              0.0f, 0.0f, 0.0f,
-        0.0f, layer_green_scale_,              0.0f, 0.0f, 0.0f,
-        0.0f,               0.0f, layer_blue_scale_, 0.0f, 0.0f,
-        0.0f,               0.0f,              0.0f, 1.0f, 0.0f
-    };
-    filters.Append(cc::FilterOperation::CreateColorMatrixFilter(color_matrix));
-  }
   if (layer_inverted_)
     filters.Append(cc::FilterOperation::CreateInvertFilter(1.0));
   if (layer_blur_sigma_) {
@@ -555,7 +539,7 @@ bool Layer::IsDrawn() const {
   const Layer* layer = this;
   while (layer && layer->visible_)
     layer = layer->parent_;
-  return layer == NULL;
+  return layer == nullptr;
 }
 
 bool Layer::ShouldDraw() const {
@@ -565,7 +549,7 @@ bool Layer::ShouldDraw() const {
 // static
 void Layer::ConvertPointToLayer(const Layer* source,
                                 const Layer* target,
-                                gfx::Point* point) {
+                                gfx::PointF* point) {
   if (source == target)
     return;
 
@@ -622,22 +606,23 @@ void Layer::SwitchToLayer(scoped_refptr<cc::Layer> new_layer) {
   if (cc_layer_->parent()) {
     cc_layer_->parent()->ReplaceChild(cc_layer_, new_layer);
   }
-  cc_layer_->SetLayerClient(NULL);
+  cc_layer_->SetLayerClient(nullptr);
   new_layer->SetOpacity(cc_layer_->opacity());
   new_layer->SetTransform(cc_layer_->transform());
   new_layer->SetPosition(cc_layer_->position());
   new_layer->SetBackgroundColor(cc_layer_->background_color());
   new_layer->SetCacheRenderSurface(cc_layer_->cache_render_surface());
+  new_layer->SetTrilinearFiltering(cc_layer_->trilinear_filtering());
 
   cc_layer_ = new_layer.get();
-  content_layer_ = NULL;
-  solid_color_layer_ = NULL;
-  texture_layer_ = NULL;
-  surface_layer_ = NULL;
+  content_layer_ = nullptr;
+  solid_color_layer_ = nullptr;
+  texture_layer_ = nullptr;
+  surface_layer_ = nullptr;
 
-  for (size_t i = 0; i < children_.size(); ++i) {
-    DCHECK(children_[i]->cc_layer_);
-    cc_layer_->AddChild(children_[i]->cc_layer_);
+  for (auto* child : children_) {
+    DCHECK(child->cc_layer_);
+    cc_layer_->AddChild(child->cc_layer_);
   }
   cc_layer_->SetLayerClient(this);
   cc_layer_->SetTransformOrigin(gfx::Point3F());
@@ -661,16 +646,69 @@ void Layer::SwitchCCLayerForTest() {
 // which could be a supprise. But we want to preserve it after switching to a
 // new cc::Layer. There could be a whole subtree and the root changed, but does
 // not mean we want to treat the cache all different.
-void Layer::SetCacheRenderSurface(bool cache_render_surface) {
-  cc_layer_->SetCacheRenderSurface(cache_render_surface);
+void Layer::AddCacheRenderSurfaceRequest() {
+  ++cache_render_surface_requests_;
+  TRACE_COUNTER_ID1("ui", "CacheRenderSurfaceRequests", this,
+                    cache_render_surface_requests_);
+  if (cache_render_surface_requests_ == 1)
+    cc_layer_->SetCacheRenderSurface(true);
 }
 
-void Layer::SetTextureMailbox(
-    const viz::TextureMailbox& mailbox,
-    std::unique_ptr<cc::SingleReleaseCallback> release_callback,
+void Layer::RemoveCacheRenderSurfaceRequest() {
+  DCHECK_GT(cache_render_surface_requests_, 0u);
+
+  --cache_render_surface_requests_;
+  TRACE_COUNTER_ID1("ui", "CacheRenderSurfaceRequests", this,
+                    cache_render_surface_requests_);
+  if (cache_render_surface_requests_ == 0)
+    cc_layer_->SetCacheRenderSurface(false);
+}
+
+void Layer::AddDeferredPaintRequest() {
+  ++deferred_paint_requests_;
+  TRACE_COUNTER_ID1("ui", "DeferredPaintRequests", this,
+                    deferred_paint_requests_);
+}
+
+void Layer::RemoveDeferredPaintRequest() {
+  DCHECK_GT(deferred_paint_requests_, 0u);
+
+  --deferred_paint_requests_;
+  TRACE_COUNTER_ID1("ui", "DeferredPaintRequests", this,
+                    deferred_paint_requests_);
+  if (!deferred_paint_requests_ && !damaged_region_.IsEmpty())
+    ScheduleDraw();
+}
+
+// Note: The code that sets this flag would be responsible to unset it on that
+// ui::Layer. We do not want to clone this flag to a cloned layer by accident,
+// which could be a supprise. But we want to preserve it after switching to a
+// new cc::Layer. There could be a whole subtree and the root changed, but does
+// not mean we want to treat the trilinear filtering all different.
+void Layer::AddTrilinearFilteringRequest() {
+  ++trilinear_filtering_request_;
+  TRACE_COUNTER_ID1("ui", "TrilinearFilteringRequests", this,
+                    trilinear_filtering_request_);
+  if (trilinear_filtering_request_ == 1)
+    cc_layer_->SetTrilinearFiltering(true);
+}
+
+void Layer::RemoveTrilinearFilteringRequest() {
+  DCHECK_GT(trilinear_filtering_request_, 0u);
+
+  --trilinear_filtering_request_;
+  TRACE_COUNTER_ID1("ui", "TrilinearFilteringRequests", this,
+                    trilinear_filtering_request_);
+  if (trilinear_filtering_request_ == 0)
+    cc_layer_->SetTrilinearFiltering(false);
+}
+
+void Layer::SetTransferableResource(
+    const viz::TransferableResource& resource,
+    std::unique_ptr<viz::SingleReleaseCallback> release_callback,
     gfx::Size texture_size_in_dip) {
   DCHECK(type_ == LAYER_TEXTURED || type_ == LAYER_SOLID_COLOR);
-  DCHECK(mailbox.IsValid());
+  DCHECK(!resource.mailbox_holder.mailbox.IsZero());
   DCHECK(release_callback);
   if (!texture_layer_.get()) {
     scoped_refptr<cc::TextureLayer> new_layer =
@@ -682,10 +720,10 @@ void Layer::SetTextureMailbox(
     // the frame_size_in_dip_ was for a previous (different) |texture_layer_|.
     frame_size_in_dip_ = gfx::Size();
   }
-  if (mailbox_release_callback_)
-    mailbox_release_callback_->Run(gpu::SyncToken(), false);
-  mailbox_release_callback_ = std::move(release_callback);
-  mailbox_ = mailbox;
+  if (transfer_release_callback_)
+    transfer_release_callback_->Run(gpu::SyncToken(), false);
+  transfer_release_callback_ = std::move(release_callback);
+  transfer_resource_ = resource;
   SetTextureSize(texture_size_in_dip);
 }
 
@@ -709,7 +747,9 @@ bool Layer::TextureFlipped() const {
 }
 
 void Layer::SetShowPrimarySurface(
-    const viz::SurfaceInfo& surface_info,
+    const viz::SurfaceId& surface_id,
+    const gfx::Size& frame_size_in_dip,
+    SkColor default_background_color,
     scoped_refptr<viz::SurfaceReferenceFactory> ref_factory) {
   DCHECK(type_ == LAYER_TEXTURED || type_ == LAYER_SOLID_COLOR);
 
@@ -720,30 +760,37 @@ void Layer::SetShowPrimarySurface(
     surface_layer_ = new_layer;
   }
 
-  surface_layer_->SetPrimarySurfaceInfo(surface_info);
+  surface_layer_->SetPrimarySurfaceId(surface_id, base::nullopt);
+  surface_layer_->SetBackgroundColor(default_background_color);
 
-  frame_size_in_dip_ = gfx::ConvertSizeToDIP(surface_info.device_scale_factor(),
-                                             surface_info.size_in_pixels());
+  frame_size_in_dip_ = frame_size_in_dip;
   RecomputeDrawsContentAndUVRect();
 
-  for (const auto& mirror : mirrors_)
-    mirror->dest()->SetShowPrimarySurface(surface_info, ref_factory);
+  for (const auto& mirror : mirrors_) {
+    mirror->dest()->SetShowPrimarySurface(
+        surface_id, frame_size_in_dip, default_background_color, ref_factory);
+  }
 }
 
-void Layer::SetFallbackSurface(const viz::SurfaceInfo& surface_info) {
+void Layer::SetFallbackSurfaceId(const viz::SurfaceId& surface_id) {
   DCHECK(type_ == LAYER_TEXTURED || type_ == LAYER_SOLID_COLOR);
   DCHECK(surface_layer_);
 
-  // TODO(fsamuel): We should compute the gutter in the display compositor.
-  surface_layer_->SetFallbackSurfaceInfo(surface_info);
+  surface_layer_->SetFallbackSurfaceId(surface_id);
 
   for (const auto& mirror : mirrors_)
-    mirror->dest()->SetFallbackSurface(surface_info);
+    mirror->dest()->SetFallbackSurfaceId(surface_id);
 }
 
-const viz::SurfaceInfo* Layer::GetFallbackSurfaceInfo() const {
+const viz::SurfaceId* Layer::GetPrimarySurfaceId() const {
   if (surface_layer_)
-    return &surface_layer_->fallback_surface_info();
+    return &surface_layer_->primary_surface_id();
+  return nullptr;
+}
+
+const viz::SurfaceId* Layer::GetFallbackSurfaceId() const {
+  if (surface_layer_)
+    return &surface_layer_->fallback_surface_id();
   return nullptr;
 }
 
@@ -757,10 +804,10 @@ void Layer::SetShowSolidColorContent() {
   SwitchToLayer(new_layer);
   solid_color_layer_ = new_layer;
 
-  mailbox_ = viz::TextureMailbox();
-  if (mailbox_release_callback_) {
-    mailbox_release_callback_->Run(gpu::SyncToken(), false);
-    mailbox_release_callback_.reset();
+  transfer_resource_ = viz::TransferableResource();
+  if (transfer_release_callback_) {
+    transfer_release_callback_->Run(gpu::SyncToken(), false);
+    transfer_release_callback_.reset();
   }
   RecomputeDrawsContentAndUVRect();
 }
@@ -812,17 +859,19 @@ SkColor Layer::background_color() const {
 }
 
 bool Layer::SchedulePaint(const gfx::Rect& invalid_rect) {
-  if ((type_ == LAYER_SOLID_COLOR && !texture_layer_.get()) ||
-      type_ == LAYER_NINE_PATCH || (!delegate_ && !mailbox_.IsValid()))
+  if (type_ == LAYER_SOLID_COLOR && !texture_layer_)
+    return false;
+  if (type_ == LAYER_NINE_PATCH)
+    return false;
+  if (!delegate_ && transfer_resource_.mailbox_holder.mailbox.IsZero())
     return false;
 
   damaged_region_.Union(invalid_rect);
-  ScheduleDraw();
-
-  if (layer_mask_) {
+  if (layer_mask_)
     layer_mask_->damaged_region_.Union(invalid_rect);
-    layer_mask_->ScheduleDraw();
-  }
+
+  if (!content_layer_ || !deferred_paint_requests_)
+    ScheduleDraw();
   return true;
 }
 
@@ -835,11 +884,13 @@ void Layer::ScheduleDraw() {
 void Layer::SendDamagedRects() {
   if (damaged_region_.IsEmpty())
     return;
-  if (!delegate_ && !mailbox_.IsValid())
+  if (!delegate_ && transfer_resource_.mailbox_holder.mailbox.IsZero())
+    return;
+  if (content_layer_ && deferred_paint_requests_)
     return;
 
-  for (cc::Region::Iterator iter(damaged_region_); iter.has_rect(); iter.next())
-    cc_layer_->SetNeedsDisplayRect(iter.rect());
+  for (gfx::Rect damaged_rect : damaged_region_)
+    cc_layer_->SetNeedsDisplayRect(damaged_rect);
   if (layer_mask_)
     layer_mask_->SendDamagedRects();
 
@@ -862,9 +913,9 @@ void Layer::CompleteAllAnimations() {
 void Layer::SuppressPaint() {
   if (!delegate_)
     return;
-  delegate_ = NULL;
-  for (size_t i = 0; i < children_.size(); ++i)
-    children_[i]->SuppressPaint();
+  delegate_ = nullptr;
+  for (auto* child : children_)
+    child->SuppressPaint();
 }
 
 void Layer::OnDeviceScaleFactorChanged(float device_scale_factor) {
@@ -872,6 +923,7 @@ void Layer::OnDeviceScaleFactorChanged(float device_scale_factor) {
     return;
   if (animator_)
     animator_->StopAnimatingProperty(LayerAnimationElement::TRANSFORM);
+  const float old_device_scale_factor = device_scale_factor_;
   device_scale_factor_ = device_scale_factor;
   RecomputeDrawsContentAndUVRect();
   RecomputePosition();
@@ -881,22 +933,19 @@ void Layer::OnDeviceScaleFactorChanged(float device_scale_factor) {
     UpdateNinePatchLayerAperture(nine_patch_layer_aperture_);
   }
   SchedulePaint(gfx::Rect(bounds_.size()));
-  if (delegate_)
-    delegate_->OnDeviceScaleFactorChanged(device_scale_factor);
-  for (size_t i = 0; i < children_.size(); ++i)
-    children_[i]->OnDeviceScaleFactorChanged(device_scale_factor);
+  if (delegate_) {
+    delegate_->OnDeviceScaleFactorChanged(old_device_scale_factor,
+                                          device_scale_factor);
+  }
+  for (auto* child : children_)
+    child->OnDeviceScaleFactorChanged(device_scale_factor);
   if (layer_mask_)
     layer_mask_->OnDeviceScaleFactorChanged(device_scale_factor);
 }
 
-void Layer::OnDelegatedFrameDamage(const gfx::Rect& damage_rect_in_dip) {
-  DCHECK(surface_layer_.get());
-  if (delegate_)
-    delegate_->OnDelegatedFrameDamage(damage_rect_in_dip);
-}
-
 void Layer::SetDidScrollCallback(
-    base::Callback<void(const gfx::ScrollOffset&)> callback) {
+    base::Callback<void(const gfx::ScrollOffset&, const cc::ElementId&)>
+        callback) {
   cc_layer_->set_did_scroll_callback(std::move(callback));
 }
 
@@ -927,7 +976,7 @@ void Layer::SetScrollOffset(const gfx::ScrollOffset& offset) {
 }
 
 void Layer::RequestCopyOfOutput(
-    std::unique_ptr<cc::CopyOutputRequest> request) {
+    std::unique_ptr<viz::CopyOutputRequest> request) {
   cc_layer_->RequestCopyOfOutput(std::move(request));
 }
 
@@ -942,10 +991,11 @@ scoped_refptr<cc::DisplayItemList> Layer::PaintContentsToDisplayList(
   gfx::Rect invalidation(
       gfx::IntersectRects(paint_region_.bounds(), local_bounds));
   paint_region_.Clear();
-  auto display_list = make_scoped_refptr(new cc::DisplayItemList);
+  auto display_list = base::MakeRefCounted<cc::DisplayItemList>();
   if (delegate_) {
-    delegate_->OnPaintLayer(
-        PaintContext(display_list.get(), device_scale_factor_, invalidation));
+    delegate_->OnPaintLayer(PaintContext(display_list.get(),
+                                         device_scale_factor_, invalidation,
+                                         GetCompositor()->is_pixel_canvas()));
   }
   display_list->Finalize();
   // TODO(domlaskowski): Move mirror invalidation to Layer::SchedulePaint.
@@ -962,13 +1012,13 @@ size_t Layer::GetApproximateUnsharedMemoryUsage() const {
   return 0;
 }
 
-bool Layer::PrepareTextureMailbox(
-    viz::TextureMailbox* mailbox,
-    std::unique_ptr<cc::SingleReleaseCallback>* release_callback) {
-  if (!mailbox_release_callback_)
+bool Layer::PrepareTransferableResource(
+    viz::TransferableResource* resource,
+    std::unique_ptr<viz::SingleReleaseCallback>* release_callback) {
+  if (!transfer_release_callback_)
     return false;
-  *mailbox = mailbox_;
-  *release_callback = std::move(mailbox_release_callback_);
+  *resource = transfer_resource_;
+  *release_callback = std::move(transfer_release_callback_);
   return true;
 }
 
@@ -1029,30 +1079,30 @@ void Layer::StackRelativeTo(Layer* child, Layer* other, bool above) {
 }
 
 bool Layer::ConvertPointForAncestor(const Layer* ancestor,
-                                    gfx::Point* point) const {
+                                    gfx::PointF* point) const {
   gfx::Transform transform;
   bool result = GetTargetTransformRelativeTo(ancestor, &transform);
-  auto p = gfx::Point3F(gfx::PointF(*point));
+  auto p = gfx::Point3F(*point);
   transform.TransformPoint(&p);
-  *point = gfx::ToFlooredPoint(p.AsPointF());
+  *point = p.AsPointF();
   return result;
 }
 
 bool Layer::ConvertPointFromAncestor(const Layer* ancestor,
-                                     gfx::Point* point) const {
+                                     gfx::PointF* point) const {
   gfx::Transform transform;
   bool result = GetTargetTransformRelativeTo(ancestor, &transform);
-  auto p = gfx::Point3F(gfx::PointF(*point));
+  auto p = gfx::Point3F(*point);
   transform.TransformPointReverse(&p);
-  *point = gfx::ToFlooredPoint(p.AsPointF());
+  *point = p.AsPointF();
   return result;
 }
 
-void Layer::SetBoundsFromAnimation(const gfx::Rect& bounds) {
+void Layer::SetBoundsFromAnimation(const gfx::Rect& bounds,
+                                   PropertyChangeReason reason) {
   if (bounds == bounds_)
     return;
 
-  base::Closure closure;
   const gfx::Rect old_bounds = bounds_;
   bounds_ = bounds;
 
@@ -1060,7 +1110,7 @@ void Layer::SetBoundsFromAnimation(const gfx::Rect& bounds) {
   RecomputePosition();
 
   if (delegate_)
-    delegate_->OnLayerBoundsChanged(old_bounds);
+    delegate_->OnLayerBoundsChanged(old_bounds, reason);
 
   if (bounds.size() == old_bounds.size()) {
     // Don't schedule a draw if we're invisible. We'll schedule one
@@ -1078,16 +1128,23 @@ void Layer::SetBoundsFromAnimation(const gfx::Rect& bounds) {
   }
 }
 
-void Layer::SetTransformFromAnimation(const gfx::Transform& transform) {
+void Layer::SetTransformFromAnimation(const gfx::Transform& transform,
+                                      PropertyChangeReason reason) {
   cc_layer_->SetTransform(transform);
+  if (delegate_)
+    delegate_->OnLayerTransformed(reason);
 }
 
-void Layer::SetOpacityFromAnimation(float opacity) {
+void Layer::SetOpacityFromAnimation(float opacity,
+                                    PropertyChangeReason reason) {
   cc_layer_->SetOpacity(opacity);
+  if (delegate_)
+    delegate_->OnLayerOpacityChanged(reason);
   ScheduleDraw();
 }
 
-void Layer::SetVisibilityFromAnimation(bool visible) {
+void Layer::SetVisibilityFromAnimation(bool visible,
+                                       PropertyChangeReason reason) {
   if (visible_ == visible)
     return;
 
@@ -1095,33 +1152,24 @@ void Layer::SetVisibilityFromAnimation(bool visible) {
   cc_layer_->SetHideLayerAndSubtree(!visible_);
 }
 
-void Layer::SetBrightnessFromAnimation(float brightness) {
+void Layer::SetBrightnessFromAnimation(float brightness,
+                                       PropertyChangeReason reason) {
   layer_brightness_ = brightness;
   SetLayerFilters();
 }
 
-void Layer::SetGrayscaleFromAnimation(float grayscale) {
+void Layer::SetGrayscaleFromAnimation(float grayscale,
+                                      PropertyChangeReason reason) {
   layer_grayscale_ = grayscale;
   SetLayerFilters();
 }
 
-void Layer::SetColorFromAnimation(SkColor color) {
+void Layer::SetColorFromAnimation(SkColor color, PropertyChangeReason reason) {
   DCHECK_EQ(type_, LAYER_SOLID_COLOR);
   cc_layer_->SetBackgroundColor(color);
   SetFillsBoundsOpaquely(SkColorGetA(color) == 0xFF);
 }
 
-void Layer::SetTemperatureFromAnimation(float temperature) {
-  layer_temperature_ = temperature;
-
-  // If we only tone down the blue scale, the screen will look very green so we
-  // also need to tone down the green, but with a less value compared to the
-  // blue scale to avoid making things look very red.
-  layer_blue_scale_ = 1.0f - temperature;
-  layer_green_scale_ = 1.0f - 0.3f * temperature;
-  SetLayerFilters();
-}
-
 void Layer::ScheduleDrawForAnimation() {
   ScheduleDraw();
 }
@@ -1158,17 +1206,13 @@ SkColor Layer::GetColorForAnimation() const {
       solid_color_layer_->background_color() : SK_ColorBLACK;
 }
 
-float Layer::GetTemperatureFromAnimation() const {
-  return layer_temperature_;
-}
-
 float Layer::GetDeviceScaleFactor() const {
   return device_scale_factor_;
 }
 
 LayerAnimatorCollection* Layer::GetLayerAnimatorCollection() {
   Compositor* compositor = GetCompositor();
-  return compositor ? compositor->layer_animator_collection() : NULL;
+  return compositor ? compositor->layer_animator_collection() : nullptr;
 }
 
 int Layer::GetFrameNumber() const {
@@ -1181,6 +1225,10 @@ float Layer::GetRefreshRate() const {
   return compositor ? compositor->refresh_rate() : 60.0;
 }
 
+ui::Layer* Layer::GetLayer() {
+  return this;
+}
+
 cc::Layer* Layer::GetCcLayer() const {
   return cc_layer_;
 }
@@ -1209,10 +1257,6 @@ void Layer::CreateCcLayer() {
   RecomputePosition();
 }
 
-gfx::Transform Layer::transform() const {
-  return cc_layer_->transform();
-}
-
 void Layer::RecomputeDrawsContentAndUVRect() {
   DCHECK(cc_layer_);
   gfx::Size size(bounds_.size());
