@@ -8,6 +8,7 @@
 
 #include <utility>
 
+#include "base/files/platform_file.h"
 #include "base/macros.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "ui/ozone/platform/drm/gpu/crtc_controller.h"
@@ -21,8 +22,7 @@
 
 namespace {
 
-void EmptySwapCallback(gfx::SwapResult) {
-}
+void EmptySwapCallback(gfx::SwapResult, const gfx::PresentationFeedback&) {}
 
 // Create a basic mode for a 6x4 screen.
 const drmModeModeInfo kDefaultMode =
@@ -503,7 +503,8 @@ TEST_F(ScreenManagerTest, EnableControllerWhenWindowHasBuffer) {
   scoped_refptr<ui::ScanoutBuffer> buffer = buffer_generator_->Create(
       drm_, DRM_FORMAT_XRGB8888, GetPrimaryBounds().size());
   window->SchedulePageFlip(
-      std::vector<ui::OverlayPlane>(1, ui::OverlayPlane(buffer)),
+      std::vector<ui::OverlayPlane>(
+          1, ui::OverlayPlane(buffer, base::kInvalidPlatformFile)),
       base::Bind(&EmptySwapCallback));
   screen_manager_->AddWindow(1, std::move(window));
 
@@ -529,7 +530,8 @@ TEST_F(ScreenManagerTest, RejectBufferWithIncompatibleModifiers) {
                                             GetPrimaryBounds().size());
 
   window->SchedulePageFlip(
-      std::vector<ui::OverlayPlane>(1, ui::OverlayPlane(buffer)),
+      std::vector<ui::OverlayPlane>(
+          1, ui::OverlayPlane(buffer, base::kInvalidPlatformFile)),
       base::Bind(&EmptySwapCallback));
   screen_manager_->AddWindow(1, std::move(window));
 
