@@ -32,7 +32,7 @@
 #include "ui/views/animation/ink_drop_highlight.h"
 #include "ui/views/animation/ink_drop_impl.h"
 #include "ui/views/background.h"
-#include "ui/views/controls/button/custom_button.h"
+#include "ui/views/controls/button/button.h"
 #include "ui/views/controls/button/label_button.h"
 #include "ui/views/controls/combobox/combobox_listener.h"
 #include "ui/views/controls/focus_ring.h"
@@ -41,6 +41,7 @@
 #include "ui/views/controls/menu/menu_runner.h"
 #include "ui/views/controls/prefix_selector.h"
 #include "ui/views/controls/textfield/textfield.h"
+#include "ui/views/layout/layout_provider.h"
 #include "ui/views/mouse_constants.h"
 #include "ui/views/painter.h"
 #include "ui/views/resources/grit/views_resources.h"
@@ -96,10 +97,10 @@ bool UseMd() {
   return ui::MaterialDesignController::IsSecondaryUiMaterial();
 }
 
-SkColor GetTextColorForEnableState(bool enabled, ui::NativeTheme* theme) {
-  return style::GetColor(style::CONTEXT_TEXTFIELD,
-                         enabled ? style::STYLE_PRIMARY : style::STYLE_DISABLED,
-                         theme);
+SkColor GetTextColorForEnableState(const Combobox& combobox, bool enabled) {
+  return style::GetColor(
+      combobox, style::CONTEXT_TEXTFIELD,
+      enabled ? style::STYLE_PRIMARY : style::STYLE_DISABLED);
 }
 
 gfx::Rect PositionArrowWithinContainer(const gfx::Rect& container_bounds,
@@ -119,10 +120,10 @@ gfx::Rect PositionArrowWithinContainer(const gfx::Rect& container_bounds,
 }
 
 // The transparent button which holds a button state but is not rendered.
-class TransparentButton : public CustomButton {
+class TransparentButton : public Button {
  public:
   TransparentButton(ButtonListener* listener, bool animate_state_change)
-      : CustomButton(listener) {
+      : Button(listener) {
     set_animate_on_state_change(animate_state_change);
     if (animate_state_change)
       SetAnimationDuration(LabelButton::kHoverAnimationDurationMs);
@@ -142,7 +143,7 @@ class TransparentButton : public CustomButton {
     // platforms they do.
     parent()->RequestFocus();
 #endif
-    return CustomButton::OnMousePressed(mouse_event);
+    return Button::OnMousePressed(mouse_event);
   }
 
   double GetAnimationValue() const {
@@ -487,6 +488,7 @@ void Combobox::ModelChanged() {
 
   content_size_ = GetContentSize();
   PreferredSizeChanged();
+  SchedulePaint();
 }
 
 void Combobox::SetSelectedIndex(int index) {
@@ -515,6 +517,12 @@ bool Combobox::SelectValue(const base::string16& value) {
   return false;
 }
 
+void Combobox::SetTooltipText(const base::string16& tooltip_text) {
+  arrow_button_->SetTooltipText(tooltip_text);
+  if (accessible_name_.empty())
+    accessible_name_ = tooltip_text;
+}
+
 void Combobox::SetAccessibleName(const base::string16& name) {
   accessible_name_ = name;
 }
@@ -595,10 +603,10 @@ gfx::Size Combobox::CalculatePreferredSize() const {
   // The preferred size will drive the local bounds which in turn is used to set
   // the minimum width for the dropdown list.
   gfx::Insets insets = GetInsets();
-  insets += gfx::Insets(Textfield::kTextPadding,
-                        Textfield::kTextPadding,
-                        Textfield::kTextPadding,
-                        Textfield::kTextPadding);
+  const LayoutProvider* provider = LayoutProvider::Get();
+  insets += gfx::Insets(
+      provider->GetDistanceMetric(DISTANCE_CONTROL_VERTICAL_TEXT_PADDING),
+      provider->GetDistanceMetric(DISTANCE_TEXTFIELD_HORIZONTAL_TEXT_PADDING));
   int total_width = std::max(kMinComboboxWidth, content_size_.width()) +
                     insets.width() + GetArrowContainerWidth();
   return gfx::Size(total_width, content_size_.height() + insets.height());
@@ -828,7 +836,8 @@ void Combobox::AdjustBoundsForRTLUI(gfx::Rect* rect) const {
 
 void Combobox::PaintText(gfx::Canvas* canvas) {
   gfx::Insets insets = GetInsets();
-  insets += gfx::Insets(0, Textfield::kTextPadding, 0, Textfield::kTextPadding);
+  insets += gfx::Insets(0, LayoutProvider::Get()->GetDistanceMetric(
+                               DISTANCE_TEXTFIELD_HORIZONTAL_TEXT_PADDING));
 
   gfx::ScopedCanvas scoped_canvas(canvas);
   canvas->ClipRect(GetContentsBounds());
@@ -836,7 +845,7 @@ void Combobox::PaintText(gfx::Canvas* canvas) {
   int x = insets.left();
   int y = insets.top();
   int text_height = height() - insets.height();
-  SkColor text_color = GetTextColorForEnableState(enabled(), GetNativeTheme());
+  SkColor text_color = GetTextColorForEnableState(*this, enabled());
   DCHECK_GE(selected_index_, 0);
   DCHECK_LT(selected_index_, model()->GetItemCount());
   if (selected_index_ < 0 || selected_index_ > model()->GetItemCount())
@@ -879,7 +888,7 @@ void Combobox::PaintText(gfx::Canvas* canvas) {
     path.rLineTo(height, -height);
     path.close();
     cc::PaintFlags flags;
-    SkColor arrow_color = GetTextColorForEnableState(true, GetNativeTheme());
+    SkColor arrow_color = GetTextColorForEnableState(*this, true);
     if (!enabled())
       arrow_color = SkColorSetA(arrow_color, gfx::kDisabledControlAlpha);
     flags.setColor(arrow_color);
@@ -987,7 +996,7 @@ void Combobox::OnMenuClosed(Button::ButtonState original_button_state) {
 }
 
 void Combobox::OnPerformAction() {
-  NotifyAccessibilityEvent(ui::AX_EVENT_VALUE_CHANGED, false);
+  NotifyAccessibilityEvent(ui::AX_EVENT_VALUE_CHANGED, true);
   SchedulePaint();
 
   // This combobox may be deleted by the listener.
