@@ -5,7 +5,6 @@
 #include "ui/views/mus/desktop_window_tree_host_mus.h"
 
 #include "base/memory/ptr_util.h"
-#include "base/message_loop/message_loop.h"
 #include "base/run_loop.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "ui/aura/client/aura_constants.h"
@@ -212,7 +211,6 @@ DesktopWindowTreeHostMus::DesktopWindowTreeHostMus(
   MusClient::Get()->AddObserver(this);
   MusClient::Get()->window_tree_client()->focus_synchronizer()->AddObserver(
       this);
-  native_widget_delegate_->AsWidget()->AddObserver(this);
   desktop_native_widget_aura_->content_window()->AddObserver(this);
   // DesktopNativeWidgetAura registers the association between |content_window_|
   // and Widget, but code may also want to go from the root (window()) to the
@@ -229,7 +227,6 @@ DesktopWindowTreeHostMus::~DesktopWindowTreeHostMus() {
   // |cursor_manager_| is destroyed.
   aura::client::SetCursorClient(window(), nullptr);
   desktop_native_widget_aura_->content_window()->RemoveObserver(this);
-  native_widget_delegate_->AsWidget()->RemoveObserver(this);
   MusClient::Get()->RemoveObserver(this);
   MusClient::Get()->window_tree_client()->focus_synchronizer()->RemoveObserver(
       this);
@@ -256,7 +253,7 @@ void DesktopWindowTreeHostMus::SendClientAreaToServer() {
 
 void DesktopWindowTreeHostMus::SendHitTestMaskToServer() {
   if (!native_widget_delegate_->HasHitTestMask()) {
-    SetHitTestMask(base::nullopt);
+    aura::WindowPortMus::Get(window())->SetHitTestMask(base::nullopt);
     return;
   }
 
@@ -265,7 +262,15 @@ void DesktopWindowTreeHostMus::SendHitTestMaskToServer() {
   // TODO(jamescook): Use the full path for the mask.
   gfx::Rect mask_rect =
       gfx::ToEnclosingRect(gfx::SkRectToRectF(mask_path.getBounds()));
-  SetHitTestMask(mask_rect);
+  aura::WindowPortMus::Get(window())->SetHitTestMask(mask_rect);
+}
+
+bool DesktopWindowTreeHostMus::IsFocusClientInstalledOnFocusSynchronizer()
+    const {
+  return MusClient::Get()
+             ->window_tree_client()
+             ->focus_synchronizer()
+             ->active_focus_client() == aura::client::GetFocusClient(window());
 }
 
 float DesktopWindowTreeHostMus::GetScaleFactor() const {
@@ -305,8 +310,8 @@ void DesktopWindowTreeHostMus::Init(aura::Window* content_window,
   if (!params.bounds.IsEmpty())
     SetBoundsInDIP(params.bounds);
 
-  cursor_manager_ = base::MakeUnique<wm::CursorManager>(
-      base::MakeUnique<NativeCursorManagerMus>(window()));
+  cursor_manager_ = std::make_unique<wm::CursorManager>(
+      std::make_unique<NativeCursorManagerMus>(window()));
   aura::client::SetCursorClient(window(), cursor_manager_.get());
   InitHost();
 
@@ -321,8 +326,7 @@ void DesktopWindowTreeHostMus::Init(aura::Window* content_window,
   }
 
   if (!params.accept_events) {
-    aura::WindowPortMus::Get(window())->SetEventTargetingPolicy(
-        ui::mojom::EventTargetingPolicy::NONE);
+    window()->SetEventTargetingPolicy(ui::mojom::EventTargetingPolicy::NONE);
   } else {
     aura::WindowPortMus::Get(content_window)->SetCanAcceptDrops(true);
   }
@@ -340,11 +344,23 @@ void DesktopWindowTreeHostMus::OnNativeWidgetCreated(
   native_widget_delegate_->OnNativeWidgetCreated(true);
 }
 
-void DesktopWindowTreeHostMus::OnNativeWidgetActivationChanged(bool active) {
-  // In Mus, when our aura Window receives an activation signal, it can be
-  // because of remote messages. We need to forward that to Widget so that
-  // Widget can notify everyone listening for that signal.
-  native_widget_delegate_->OnNativeWidgetActivationChanged(active);
+void DesktopWindowTreeHostMus::OnActiveWindowChanged(bool active) {
+  // This function is called when there is a change in the active window the
+  // FocusClient for window() is associated with. This needs to potentially
+  // propagate to mus (the change may originate locally, not from mus).
+  // Propagating to the server is done by resetting the ActiveFocusClient.
+  if (active && !IsFocusClientInstalledOnFocusSynchronizer()) {
+    MusClient::Get()
+        ->window_tree_client()
+        ->focus_synchronizer()
+        ->SetActiveFocusClient(aura::client::GetFocusClient(window()),
+                               window());
+  } else if (!active && IsFocusClientInstalledOnFocusSynchronizer()) {
+    MusClient::Get()
+        ->window_tree_client()
+        ->focus_synchronizer()
+        ->SetActiveFocusClient(nullptr, nullptr);
+  }
 }
 
 void DesktopWindowTreeHostMus::OnWidgetInitDone() {
@@ -359,7 +375,7 @@ void DesktopWindowTreeHostMus::OnWidgetInitDone() {
 }
 
 std::unique_ptr<corewm::Tooltip> DesktopWindowTreeHostMus::CreateTooltip() {
-  return base::MakeUnique<corewm::TooltipAura>();
+  return std::make_unique<corewm::TooltipAura>();
 }
 
 std::unique_ptr<aura::client::DragDropClient>
@@ -540,7 +556,7 @@ gfx::Rect DesktopWindowTreeHostMus::GetWorkAreaBoundsInScreen() const {
 }
 
 void DesktopWindowTreeHostMus::SetShape(
-    std::unique_ptr<SkRegion> native_region) {
+    std::unique_ptr<Widget::ShapeRects> native_shape) {
   NOTIMPLEMENTED();
 }
 
@@ -560,8 +576,21 @@ void DesktopWindowTreeHostMus::Activate() {
 }
 
 void DesktopWindowTreeHostMus::Deactivate() {
-  if (is_active_)
-    DeactivateWindow();
+  if (!is_active_)
+    return;
+
+  // Reset the active focus client, which will trigger resetting active status.
+  // This is done so that we deactivate immediately.
+  DCHECK(IsFocusClientInstalledOnFocusSynchronizer());
+  MusClient::Get()
+      ->window_tree_client()
+      ->focus_synchronizer()
+      ->SetActiveFocusClient(nullptr, nullptr);
+  DCHECK(!is_active_);
+
+  // Then ask the window manager to deactivate, which effectively means pick
+  // another window to activate.
+  DeactivateWindow();
 }
 
 bool DesktopWindowTreeHostMus::IsActive() const {
@@ -632,9 +661,7 @@ Widget::MoveLoopResult DesktopWindowTreeHostMus::RunMoveLoop(
   static_cast<internal::NativeWidgetPrivate*>(
       desktop_native_widget_aura_)->ReleaseCapture();
 
-  base::MessageLoopForUI* loop = base::MessageLoopForUI::current();
-  base::MessageLoop::ScopedNestableTaskAllower allow_nested(loop);
-  base::RunLoop run_loop;
+  base::RunLoop run_loop(base::RunLoop::Type::kNestableTasksAllowed);
 
   ui::mojom::MoveLoopSource mus_source =
       source == Widget::MOVE_LOOP_SOURCE_MOUSE
@@ -750,21 +777,14 @@ void DesktopWindowTreeHostMus::OnWindowManagerFrameValuesChanged() {
   SendHitTestMaskToServer();
 }
 
-void DesktopWindowTreeHostMus::OnWidgetActivationChanged(Widget* widget,
-                                                         bool active) {
-  // TODO(erg): Theoretically, this shouldn't be necessary. We should be able
-  // to just set |is_active_| in OnNativeWidgetActivationChanged() above,
-  // instead of asking the Widget to change the activation and have the widget
-  // then tell us the activation has changed. But if we do that, focus breaks.
-  is_active_ = active;
-}
-
 void DesktopWindowTreeHostMus::OnActiveFocusClientChanged(
     aura::client::FocusClient* focus_client,
     aura::Window* focus_client_root) {
   if (focus_client_root == this->window()) {
+    is_active_ = true;
     desktop_native_widget_aura_->HandleActivationChanged(true);
   } else if (is_active_) {
+    is_active_ = false;
     desktop_native_widget_aura_->HandleActivationChanged(false);
   }
 }
@@ -794,6 +814,16 @@ void DesktopWindowTreeHostMus::ShowImpl() {
 }
 
 void DesktopWindowTreeHostMus::HideImpl() {
+  // When hiding we can't possibly be active any more. Reset the FocusClient,
+  // which effectively triggers giving up focus (and activation). Mus will
+  // eventually generate a focus event, but that's async.
+  if (IsFocusClientInstalledOnFocusSynchronizer()) {
+    MusClient::Get()
+        ->window_tree_client()
+        ->focus_synchronizer()
+        ->SetActiveFocusClient(nullptr, nullptr);
+  }
+
   native_widget_delegate_->OnNativeWidgetVisibilityChanging(false);
   WindowTreeHostMus::HideImpl();
   window()->Hide();
