@@ -68,6 +68,7 @@
 #include "content/public/common/main_function_params.h"
 #include "content/public/common/service_names.mojom.h"
 #include "content/public/common/url_constants.h"
+#include "device/geolocation/public/cpp/location_provider.h"
 #include "mojo/public/cpp/bindings/binding.h"
 #include "mojo/public/cpp/bindings/binding_set.h"
 #include "printing/features/features.h"
@@ -93,7 +94,6 @@
 #include "certificate_error_controller_p.h"
 #include "desktop_screen_qt.h"
 #include "devtools_manager_delegate_qt.h"
-#include "location_provider_qt.h"
 #include "media_capture_devices_dispatcher.h"
 #include "net/network_delegate_qt.h"
 #include "net/qrc_protocol_handler_qt.h"
@@ -123,6 +123,10 @@
 #include "renderer_host/pepper/pepper_host_factory_qt.h"
 #endif
 
+#if defined(QT_USE_POSITIONING)
+#include "location_provider_qt.h"
+#endif
+
 #include <QGuiApplication>
 #include <QLocale>
 #ifndef QT_NO_OPENGL
@@ -737,7 +741,11 @@ bool ContentBrowserClientQt::CanCreateWindow(
 
 std::unique_ptr<device::LocationProvider> ContentBrowserClientQt::OverrideSystemLocationProvider()
 {
+#if defined(QT_USE_POSITIONING)
     return base::WrapUnique(new LocationProviderQt());
+#else
+    return nullptr;
+#endif
 }
 
 scoped_refptr<net::URLRequestContextGetter> GetSystemRequestContextOnUIThread()
